package com.isu.gaswellwatch.service.impl;

import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.isu.gaswellwatch.dao.UserOperationRecordDao;
import com.isu.gaswellwatch.dao.UserOperationRecordMapper;
import com.isu.gaswellwatch.entity.UserOperationRecord;
import com.isu.gaswellwatch.enums.LogResult;
import com.isu.gaswellwatch.enums.LogType;
import com.isu.gaswellwatch.service.UserOperationRecordService;
import com.isu.gaswellwatch.thread.AsyncLogTaskFactory;
import com.isu.gaswellwatch.thread.AsyncLogThreadPoolManager;
import com.isu.gaswellwatch.utils.ConverterUtil;
import com.isu.gaswellwatch.vo.LoginRecordExportVO;
import com.isu.gaswellwatch.vo.UserOperationRecordExportVO;
import com.isu.gaswellwatch.vo.UserOperationRecordVO;
import jakarta.servlet.ServletOutputStream;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.net.URLEncoder;
import java.util.List;
import java.util.concurrent.LinkedBlockingQueue;

@Service("userOperationRecordService")
@Transactional(rollbackFor = Exception.class)
public class UserOperationRecordServiceImpl extends ServiceImpl<UserOperationRecordMapper, UserOperationRecord> implements UserOperationRecordService {
    /**
     * 操作记录批处理条数
     */
    public static int CACHE_BASE_CAPACITY = 400;
    /**
     * 操作记录缓存队列最大存储大小：CACHE_BASE_CAPACITY+CACHE_MAX_INCREMENT
     */
    public static int CACHE_MAX_INCREMENT = 2000;

    /**
     * 缓存，在插入数据库前先存入此。
     */
    public final LinkedBlockingQueue<UserOperationRecord> CACHE = new LinkedBlockingQueue<>(CACHE_BASE_CAPACITY + CACHE_MAX_INCREMENT);

    /**
     * 信号量，用于标记当前是否有任务正在执行，{@code true}表示当前无任务进行。
     */
    private volatile boolean taskFinish = true;

    @Autowired
    private UserOperationRecordDao userOperationRecordDao;

    @Override
    public void addOperationRecord(UserOperationRecord operationRecord) {
        CACHE.add(operationRecord);
        if (CACHE.size() >= CACHE_BASE_CAPACITY && taskFinish) {
            synchronized (this) {
                if(taskFinish){
                    try {
                        AsyncLogThreadPoolManager.getInstance().execute(new AsyncLogTaskFactory(this).batchSaveRecord());
                    } catch (Exception e) {
                        e.printStackTrace();
                    } finally {
                        // 任务执行完毕后修改标志位
                        taskFinish = true;
                    }
                }
            }
        }
    }


    /**
     * 每分钟入库任务
     */
    @Scheduled(cron = "0 0/1 * * * ?")
    public void doTask() {
        if (taskFinish) {
            synchronized (this) {
                if (taskFinish) {
                    taskFinish = false;
                    try {
                        AsyncLogThreadPoolManager.getInstance().execute(new AsyncLogTaskFactory(this).batchSaveRecord());
                    } catch (Exception e) {
                        e.printStackTrace();
                    } finally {
                        // 任务执行完毕后修改标志位
                        taskFinish = true;
                    }
                }

            }
        }
    }

    @Override
    public Page<UserOperationRecordVO> page(Integer currentPage, Integer pageSize, Integer type, String username, String nickname, Integer result, String startTime, String endTime) {
        return userOperationRecordDao.page(new Page<>(currentPage,pageSize),type,username,nickname,result,startTime,endTime);
    }

    @Override
    public void export(HttpServletResponse response, Integer type, String username, String nickname, Integer result, String startTime, String endTime) {
        if(type== LogType.LOGIN.getVal()){
            exportLoginLog(response,type,username,nickname,result,startTime,endTime);
            return;
        }
        List<UserOperationRecordVO> list=userOperationRecordDao.list(type,username,nickname,result,startTime,endTime);
        List<UserOperationRecordExportVO> export = ConverterUtil.convert(list, UserOperationRecordExportVO.class);
        if(!CollectionUtils.isEmpty(export)) {
            for (UserOperationRecordExportVO userOperationRecordExportVO : export) {
                userOperationRecordExportVO.setTypeDesc(LogType.getDescription(userOperationRecordExportVO.getType()));
                userOperationRecordExportVO.setResultDesc(LogResult.getDescription(userOperationRecordExportVO.getResult()));
            }
        }
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        String fileName = null;
        try {
            ServletOutputStream outputStream = response.getOutputStream();
            fileName = URLEncoder.encode("操作日志", "UTF-8").replaceAll("\\+", "%20");
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
            EasyExcel.write(outputStream)
                    // 这里放入动态头
                    .head(UserOperationRecordExportVO.class).sheet("data")
                    // 当然这里数据也可以用 List<List<String>> 去传入
                    .doWrite(export);
        } catch (Exception e) {
            log.error(e.getMessage());
        }
    }

    public void exportLoginLog(HttpServletResponse response, Integer type, String username, String nickname, Integer result, String startTime, String endTime) {
        List<UserOperationRecordVO> list=userOperationRecordDao.list(type,username,nickname,result,startTime,endTime);
        List<LoginRecordExportVO> export = ConverterUtil.convert(list, LoginRecordExportVO.class);
        if(!CollectionUtils.isEmpty(export)) {
            for (LoginRecordExportVO loginRecordExportVO : export) {
                loginRecordExportVO.setResultDesc(LogResult.getDescription(loginRecordExportVO.getResult()));
            }
        }
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        String fileName = null;
        try {
            ServletOutputStream outputStream = response.getOutputStream();
            fileName = URLEncoder.encode("登录日志", "UTF-8").replaceAll("\\+", "%20");
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
            EasyExcel.write(outputStream)
                    // 这里放入动态头
                    .head(LoginRecordExportVO.class).sheet("data")
                    // 当然这里数据也可以用 List<List<String>> 去传入
                    .doWrite(export);
        } catch (Exception e) {
            log.error(e.getMessage());
        }
    }
}

